function x = FormShape(c, ccum, a, P, ends)

% Extracts a shape x from the contour c using 
% the parametrisation a
% P must be 8, 16, 32 or 64. 
% a(1...P-1) denotes the relative position between parents
%  points, e.g. 16 has parents (0,32)
% The details depend on the type of shape given in ends.
%   ends=0: P+1 points given by a(1...P-1)
%   ends=1: P   points given by a(1...P)
%   ends=2: P+1 points given by a(1...P+1)
% With P=32 the open shapes (ends=0 or 2) constructs the points 0...32, 
%  and finally
%  point 0 and 32 are defined and the whole set is shifted to 1...33
%
% cumsumc contains the arc length location of the c-points 
% normalised to (i.e. divided by) the total arc length.
%
% (c) Thodberg 2003

aa = zeros(1, P+1);  % arc length coordinates of shape
aa(1) = 0;
aa(P+1) = 1;
aa(P/2+1) = a(P/2);

if P>256, error('Too many nodes, 256 is max'); end 

if P>=256 
    for k=65:128:P
        aa(k) = aa(k-64) + a(k-1)*(aa(k+64) - aa(k-64));    
end; end;

if P>=128 
    for k=33:64:P
        aa(k) = aa(k-32) + a(k-1)*(aa(k+32) - aa(k-32));    
end; end;

if P>=64 
    for k=17:32:P
        aa(k) = aa(k-16) + a(k-1)*(aa(k+16) - aa(k-16));    
end; end;

if P>=32 
    for k=9:16:P
        aa(k) = aa(k-8) + a(k-1)*(aa(k+8) - aa(k-8));    
end; end;

if P>=16 
    for k=5:8:P
        aa(k) = aa(k-4) + a(k-1)*(aa(k+4) - aa(k-4));    
end; end;

for k=3:4:P
    aa(k) = aa(k-2) + a(k-1)*(aa(k+2) - aa(k-2)); end   

for k=2:2:P
    aa(k) = aa(k-1) + a(k-1)*(aa(k+1) - aa(k-1)); end   

if ends==1  % Effect of a(P): 
              % Change mark 1 with fixed P/2+1 and linear in-between
    aa1 =   a(P) +    aa(      1:P/2    )  * (1 - a(P)/   aa(P/2+1));
    aa2 = 1+a(P) - (1-aa((P/2+1):(end-1))) * (1 + a(P)/(1-aa(P/2+1)));
    aa = mod([aa1 aa2], 1);
end
if ends==2  % The effect of a(P) and a(P+1) is to 
    %    move away from terminals:
    % Move mark 1   a(P)   away from contour start, 
    % Move mark P+1 a(P+1) away from contour end
    % Keep mark P/2+1 fixed and do linear interpolation
    if(a(P  )<0) a(P  )=0.0; end          
    if(a(P+1)<0) a(P+1)=0.0; end          
    aa1 =   a(P) +    aa(      1:P/2)  * (1 - a(P)  /   aa(P/2+1));
    aa2 = 1-a(P+1)-(1-aa((P/2+1):end)) * (1 - a(P+1)/(1-aa(P/2+1)));
    aa = [aa1 aa2];
end

x = interp1(ccum, c, aa);
