% Main script for MDL analysis of shapes
% Copyright, February 2003, Thodberg, IMM, DTU
% (It is a script rather than a function in order to
% make the script create a numbr of variables in meory which can be 
% assessed for detailed analysis and to run additional iterations 
% with the script MDLmore
%--------------------------------------------------------------------
%  There are three demo scripts. 
%  Each of them sets up the informations appropriately
%      for this script and invokes it 
%  runBoxBump
%  runMetacarpal
%  runFemurFree
%
% To use the method on your data, write your own runXXX scripts  

%---- Step 1: Book-keeping ----------

disp(sprintf('Running %d passes of data %s in mode %d, cutoff %f', ...
    nPasses, studyName, mode, sigmaCut));
cutoff = sigmaCut^2;
totalPasses = 0;

Ccum  = zeros(size(C)); 
s = size(C,1);
for ss=1:s
    cdiff = C(ss,2:end) - C(ss,1:(end-1));
    result = [0,cumsum(abs(cdiff))];
    Ccum(ss,:) = result / result(end);
end

tolerance = 0.05;

% figure(11);plot(C(:,:).','-');axis equal;

s = size(C,1);     % number of shapes

% N is number of marks in PCA
if ends==1   % closed curve
    N = P;   
else
    N = P + 1;   % open curves have one extra mark
end
nps = length(ps);

%---- Step 2 ----------
% Define N1 fixpoints (at least one) on the first contour, 
% Define Nx (1<=Nx<=N1) fixpoints on the other contours

A = 0.5 * ones(s, P-1);   % open fixed end curves
if ends==1        % For closed curves add point 0=P     
    A = [A zeros(s,1)];
elseif ends==2    % for open free curves add
    A = [A Astart*ones(s,1) Aend*ones(s,1)];
end

if useMasterNodeValues  
    if ends~=1 error('useMasterValues implemtend only for ends =1'); end
    if initAll    
        A = InitNodes(A, ends, Ccum(:,MasterNodeValues));
    else 
        for ss=1:s
            A(ss,:) = InitNodes(A(ss,:), ends, Ccum(1,MasterNodeValues));
        end
    end
end

aTarget = A(1,ps);

AOrig = A;

%---- Step 3 ----------
% Perform initial shape analysis.
% Display contours and shapes
% Display modes and eigenvalue spectrum 
X = zeros(s, N);

if curvFac>0 
    NK = N;
    if(ends~=1) NK = N - 2*curvSkip; end;
    K = zeros(s, NK);
end

for ss=1:s
    X(ss,:) = FormShape(C(ss,1:Cn(ss)), Ccum(ss,1:Cn(ss)), A(ss,:), P, ends);
    if curvFac>0 K(ss,:) = Curvatures(X(ss,:), curvSmear, ends, curvSkip); end
end

% plot([X(:,:) X(:,1)].','-+'); axis equal;

for ss=1:s
    X(ss,:) = X(ss,:) - mean(X(ss,:));
    X(ss,:) = X(ss,:) / norm(X(ss,:));
end

Xa = Procrustes(X);
% figure(22); plot([Xa Xa(:,1)].','-+'); axis equal;
lambdasOrig = PCAShapes(Xa);
if curvFac>0 MDLK = curvFac*CurvCost(K); else MDLK = 0; end
MDL = DescriptionLength(lambdasOrig, cutoff); 
MDL = MDL + MDLK;
if mode==2, MDL = MDL + NodePenalty(A, ps, aTarget, tolerance); end
MDLOrig = MDL; 

[lambdas, VOrig, muOrig] = PCAShapes(Xa);

aspectRatio = myrange(real(muOrig)) / myrange(imag(muOrig));
% ShowNodes(C, Cn, Ccum, A, P, ends, aspectRatio);

% plot(muOrig,'-+'); axis equal;
% figure(88); plot(Xa(1:39,:).','-+'); axis equal;
% figure(88); plot(Xa(16:17,:).','-+'); axis equal;
% figure(88); plot(Xa(25:39,:).','-+'); axis equal;

%---- Step 4 ----------
% Perform some iterations and display result

stepsizeIni = 0.01;
stepsizes = stepsizeIni * ones(1, nps); 

if mode==1
    [A, MDL, stepsizes, mu, V, lambdas, AHis, MDLHis] = ...
        Optimise(C, Cn, Ccum, ends, nPasses, P, A, ps, nps, ...
        stepsizes, cutoff, ...
        curvSmear, curvFac, curvSkip);
else
    aTargetNow = aTarget;
    [A, MDL, stepsizes, aTargetNow, mu, V, lambdas, AHis, MDLHis] = ...
        OptimiseCon(C, Cn, Ccum, ends, nPasses, P, A, ps, nps, ...
        stepsizes, aTargetNow, cutoff, mode, aTarget, ...
        tolerance, curvSmear, curvFac, curvSkip);
end

totalPasses = totalPasses + nPasses
AHisTot = [reshape(AOrig(:,ps), [s 1 nps]) AHis];
MDLHisTot = [MDLOrig MDLHis];

Figures(muOrig, VOrig, lambdasOrig, mu, V, lambdas, ...
    cutoff, ps, P, ...
    AHisTot, MDLHisTot, studyName, aspectRatio);
ShowNodes(C, Cn, Ccum, A, P, ends, aspectRatio);
if curvFac>0 ShowCurv(C, Cn, Ccum, A, P, ends, curvSmear, curvSkip); end
if curvFac>0 ShowExtremes(C, Cn, Ccum, A, P, ends, ...
        curvSmear, curvSkip, mu); end

%---------------------------------------------------- end---

